// naumann@stce.rwth-aachen.de

#include <Eigen/Dense>
#include "dco.hpp"

#include <cmath>
using namespace std;

template<typename T>
T f(T p, T x) {
  return exp(sin(p*pow(x,2)));
}

template<typename T>
T df_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, yTT;
  xTT=x;
  dco::derivative(xTT)=1;
  yTT=f<TT>(p,xTT);
  return dco::derivative(yTT);
}

template<typename T>
T ddf_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, dfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  dfTT=df_dx<TT>(p,xTT);
  return dco::derivative(dfTT);
}

template<typename T>
T bisection(T p, T x) {
  T x_up=x+10; 
  x=x-10; 
  while (fabs(x-x_up)>1e-7) {
    T x_mid=(x_up+x)/2; 
    T dydx=df_dx(p,x);
    T dydx_mid=df_dx(p,x_mid);
    if (fabs(dydx_mid)<1e-7) { 
      x=x_up=x_mid;
    }
    else if ((dydx<=0&&dydx_mid<=0)||(dydx>=0&&dydx_mid>=0)) {
      x=x_mid; 
    } else {
      x_up=x_mid; 
    }
    cout << "dfdx(" << x_mid << ")=" << df_dx(p,x) << endl;
  }
  return x;
}

int main() {
  using T=double;
  T p=1, x=1;
  cout << "y=" << f(p,x) << endl;
  x=bisection(p,x);
  cout << "x=" << x << endl;
  cout << "y=" << f(p,x) << endl;
  cout << "dfdx=" << df_dx(p,x) << endl;
  cout << "ddfdxx=" << ddf_dx_dx(p,x) << endl;
  return 0;
}
